@@ -59,15 +59,14 @@ class PublishFileMessage(FileOperationEndpoint, TimeTokenOverrideMixin):
             return message
 
     def _build_message(self):
-        return self._encrypt_message(
-            {
-                "message": self._message,
-                "file": {
-                    "id": self._file_id,
-                    "name": self._file_name
-                }
+        message = {
+            "message": self._message,
+            "file": {
+                "id": self._file_id,
+                "name": self._file_name
             }
-        )
+        }
+        return self._encrypt_message(message)
 
     def build_path(self):
         message = self._build_message()
